
/**
 * Elektricka sit, do ktere se zapojuji elektricka zarizeni.
 * Sit muze byt pod proudem (implicitni stav), ale v pripade pretizeni zapojenymi 
 * elektrickymi zarizenimi muze jistic proud vypnout. V takovem pripade je 
 * nutne opet proud zapnout pomoci metody restoreCurrent().
 * 
 * @author Ales Zlamal
 * @author Radek Oslejsek, oslejsek@fi.muni.cz
 */

public interface Circuit
{
    /**
     * Metoda simuluje cinnost jistice. Jestlize je sit bez proudu, metoda nedela nic.
     * Jestlize je pod proudem, zjisti aktualni zatizeni tak, ze secte proud odebirany 
     * vsemi zapojenymi ZAPNUTYMI zarizenimi (odber proudu elektrickym zarizenim viz metoda 
     * AbstractElectricDevice.getCurrent). V pripade pretizeni (soucet je vetsi jak 
     * maximalni proud) se proud vypne.
     * <p/>
     * Pozn: V metode nedojde k atomickemu selhani. Pri pretizeni site totiz nejprve 
     * metoda musi "vypnout proud" (tj. zmenit stav instance Circuit) a pak teprve 
     * vyhodit vyjimku.
     * 
     * @throws CircuitOverloadException pokud doslo k pretizeni a jistic vypnul proud
     */
    void currentBreakerTest() throws CircuitOverloadException;
    
    /**
     * Pokus o "nahozeni jistice", tj. pokus o opetovne zapnuti proudu v siti.
     * 
     * @throws CircuitOverloadException jestlize po zapnuti proudu jistic
     *         proud opet vypnul z duvodu pretizeni
     */
    void restoreCurrent() throws CircuitOverloadException;

    /**
     * Pripojeni elektrickeho zarizeni do site (zapojeni do elektricke zasuvky).
     * Zarizeni muze zpusobit vypadek proudu pouze pokud je zapojovano do site
     * v zapnutem stavu.
     * 
     * @param device elektricke zarizeni, ktere je pripojovano do site
     * @throws IllegalArgumentException pokud je device null
     * @throws CircuitOverloadException pokud je zarizeni zapnute a jistic vypnul proud
     *         z duvodu pretizeni site
     */
    void linkTo(ElectricDevice device) throws CircuitOverloadException;

    /**
     * Odpoji elektricke zarizeni od site (vytazeni z elektricke zasuvky).
     * Pri odpojovani nemuze nikdy dojit k vypadku proudu.
     * 
     * @param device elektricke zarizeni, ktere je odpojovano ze site
     * @throws IllegalArgumentException pokud je device null, nebo pokud zarizeni
     *         neni zapojeno v teto siti
     */
    void unlink(ElectricDevice device);        
}
