
import java.util.AbstractCollection;

/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

/**
 *
 * @author misko
 */
class ElectricDevice extends AbstractElectricDevice
        implements Comparable<AbstractElectricDevice> {

    private boolean on = false;
    private Circuit circuit;

    public ElectricDevice(String name, int voltage, int wattage) {
        super(name, voltage, wattage);
    }

    /**
     * Zjisti, zda je zarizeni zapnuto (tj. jestli ma vypinac v poloze "zapnuto", bez ohledu na 
     * pripojeni k elektricke siti nebo stavu site).
     * 
     * @return true jestlize je zarizeni zapnuto, jinak false
     */
    @Override
    boolean isOn() {
        return on;
    }

    /**
     * Vypne zarizeni, tj. prepne vypinac do stavu "vypnuto". Vypnutim zarizeni nikdy
     * nemuzeme zpusobit vypadek proudu.
     */
    @Override
    void switchOff() {
        on = false;
    }

    /**
     * Zapne zarizeni, tj. prepne vypinac do stavu "zapnuto". Jestlize je zarizeni pripojeno
     * k siti, vyvola metoda test jistice v elektricke siti. I v pripade, ze jistic vypne proud
     * z duvodu pretizeni, zustava zarizeni ve stavu "zapnuto".
     * <p/>
     * Tato metoda neselze atomicky. Nejprve se zmeni stav objektu na "zapnuto",
     * pak se teprve pripadne generuje vyjimka indikujici vypadek elektrickeho proudu.
     * 
     * @throws CircuitOverloadException pokud po zapnuti zarizeni doslo k vypadku proudu v siti
     */
    @Override
    void switchOn() throws CircuitOverloadException {
        on = true;
        if (circuit != null) {
            circuit.currentBreakerTest();
        }
    }

    /**
     * Pripojeni do site (zasunuti snury do zasuvky). Jestlize je zarizeni ve stavu "zapnuto",
     * muze zapojeni zpusobit vypadek proudu. Proto metoda v takovem pripade 
     * vyvola test jistice v elektricke siti.
     * 
     * @param  circuit elektricky obvod, do ktereho se zarizeni pripojuje
     * @throws java.lang.UnsupportedOperationException pokud je jiz zarizeni pripojeno 
     *         do jine site
     * @throws IllegalArgumentException pokud je argument circuit null
     * @throws CircuitOverloadException pokud doslo po zapojeni k vypadku proudu
     */
    @Override
    void plug( Circuit circuit) throws CircuitOverloadException {
        if (circuit == null) {
            throw new IllegalArgumentException("circuit is null");
        }
        if (this.circuit != null && !this.circuit.equals(circuit)) {
            throw new UnsupportedOperationException("connected elsewhere");
        }
        this.circuit = circuit;
        circuit.linkTo(this);

    }

    /**
     * Odpojeni od site (vytahnuti snury ze zasuvky). Tato operace nemuze nikdy zpusobit
     * vypadek site.
     * 
     * @throws java.lang.UnsupportedOperationException pokud neni toto zarizeni pripojeno
     *         do elektricke site
     */
    @Override
    void unplug() {
        if (circuit == null) {
            throw new UnsupportedOperationException("not connected");
        }
        circuit.unlink(this);
        circuit = null;
    }

    public int compareTo(AbstractElectricDevice device) {
        if (device==null) {
            throw new NullPointerException("device");
        }
        return getName().compareTo(device.getName());
    }

}
